@@ -1,4 +1,4 @@
-# Copyright 2014 Google Inc.
+# Copyright 2014 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -63,10 +63,10 @@ class Test__UTC(unittest.TestCase):
         klass = self._get_target_class()
         try:
             import pytz
-        except ImportError:
+        except ImportError:  # pragma: NO COVER
             self.assertIsInstance(MUT.UTC, klass)
         else:
-            self.assertIs(MUT.UTC, pytz.UTC)  # pragma: NO COVER
+            self.assertIs(MUT.UTC, pytz.UTC)
 
     def test_dst(self):
         import datetime
@@ -77,12 +77,22 @@ class Test__UTC(unittest.TestCase):
     def test_fromutc(self):
         import datetime
 
-        naive_epoch = datetime.datetime.utcfromtimestamp(0)
+        naive_epoch = datetime.datetime(
+            1970, 1, 1, 0, 0, 1, tzinfo=None)
         self.assertIsNone(naive_epoch.tzinfo)
         tz = self._make_one()
         epoch = tz.fromutc(naive_epoch)
         self.assertEqual(epoch.tzinfo, tz)
 
+    def test_fromutc_with_tz(self):
+        import datetime
+
+        tz = self._make_one()
+        epoch_with_tz = datetime.datetime(
+            1970, 1, 1, 0, 0, 1, tzinfo=tz)
+        epoch = tz.fromutc(epoch_with_tz)
+        self.assertEqual(epoch.tzinfo, tz)
+
     def test_tzname(self):
         tz = self._make_one()
         self.assertEqual(tz.tzname(None), 'UTC')
@@ -554,6 +564,35 @@ class Test__pb_timestamp_to_datetime(unittest.TestCase):
         self.assertEqual(self._call_fut(timestamp), dt_stamp)
 
 
+class Test__from_any_pb(unittest.TestCase):
+
+    def _call_fut(self, pb_type, any_pb):
+        from google.cloud._helpers import _from_any_pb
+
+        return _from_any_pb(pb_type, any_pb)
+
+    def test_success(self):
+        from google.protobuf import any_pb2
+        from google.type import date_pb2
+
+        in_message = date_pb2.Date(year=1990)
+        in_message_any = any_pb2.Any()
+        in_message_any.Pack(in_message)
+        out_message = self._call_fut(date_pb2.Date, in_message_any)
+        self.assertEqual(in_message, out_message)
+
+    def test_failure(self, ):
+        from google.protobuf import any_pb2
+        from google.type import date_pb2
+        from google.type import timeofday_pb2
+
+        in_message = any_pb2.Any()
+        in_message.Pack(date_pb2.Date(year=1990))
+
+        with self.assertRaises(TypeError):
+            self._call_fut(timeofday_pb2.TimeOfDay, in_message)
+
+
 class Test__pb_timestamp_to_rfc3339(unittest.TestCase):
 
     def _call_fut(self, timestamp):
